package com.ibm.watson_stt;

import android.app.Activity;
import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;

import com.ibm.cloud.sdk.core.security.Authenticator;
import com.ibm.cloud.sdk.core.security.IamAuthenticator;
import com.ibm.watson.developer_cloud.android.library.audio.MicrophoneInputStream;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.ibm.watson.developer_cloud.android.library.audio.MicrophoneHelper;
import com.ibm.watson.developer_cloud.android.library.audio.utils.ContentType;
import com.ibm.watson.speech_to_text.v1.SpeechToText;
import com.ibm.watson.speech_to_text.v1.model.RecognizeOptions;
import com.ibm.watson.speech_to_text.v1.model.SpeechRecognitionResult;
import com.ibm.watson.speech_to_text.v1.model.SpeechRecognitionResults;
import com.ibm.watson.speech_to_text.v1.websocket.BaseRecognizeCallback;
import com.ibm.watson.speech_to_text.v1.websocket.RecognizeCallback;

/**
 * WatsonSttPlugin
 */
public class WatsonSttPlugin implements FlutterPlugin, MethodCallHandler, EventChannel.StreamHandler, ActivityAware {

    private class MicrophoneRecognizeDelegate extends BaseRecognizeCallback implements RecognizeCallback {
        Handler mainHandler = new Handler(Looper.getMainLooper());

        @Override
        public void onTranscription(SpeechRecognitionResults speechResults) {
            System.out.println(speechResults);
            if (speechResults.getResults() != null && !speechResults.getResults().isEmpty()) {
                final String text = speechResults.getResults().get(0).getAlternatives().get(0).getTranscript();

                SpeechRecognitionResult firstResult = speechResults.getResults().get(0);
                if (firstResult.isXFinal()) {
                    mainHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            WatsonSttPlugin.this.events.success(text);
                        }
                    });
                }
            }
        }

        @Override
        public void onError(Exception e) {
            try {
                // This is critical to avoid hangs
                // (see https://github.com/watson-developer-cloud/android-sdk/issues/59)
                capture.close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }

            mainHandler.post(new Runnable() {
                @Override
                public void run() {
                    WatsonSttPlugin.this.events.error("error", "MicrophoneRecognizeDelegate", null);
                }
            });
        }

        @Override
        public void onDisconnected() {

        }
    }

    private RecognizeOptions getRecognizeOptions(InputStream captureStream) {
        return new RecognizeOptions.Builder()
                .audio(captureStream)
                .contentType(ContentType.OPUS.toString())
                .model(speechModel)
                .interimResults(true)
                .inactivityTimeout(2000)
                .build();
    }

    private String speechModel;
    private SpeechToText speechService;
    private MicrophoneInputStream capture;
    private MicrophoneHelper microphoneHelper;
    private Activity attachedActivity;
    private EventChannel.EventSink events;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        final MethodChannel channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "method_watson_stt");
        channel.setMethodCallHandler(this);
        final EventChannel eventChannel = new EventChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "event_watson_stt");
        eventChannel.setStreamHandler(this);
    }

    private SpeechToText initSpeechToTextService(String speech_text_api_key,String speech_text_url) {
        Authenticator authenticator = new IamAuthenticator(speech_text_api_key);
        SpeechToText service = new SpeechToText(authenticator);
        service.setServiceUrl(speech_text_url);
        return service;
    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        this.attachedActivity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {
    }

    @Override
    public void onDetachedFromActivity() {
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (call.method.equals("startRecognize")) {

          Map<String,String> arguments = (Map<String, String>) call.arguments;
          String speech_text_api_key = arguments.get("apiKey");
          String speech_text_url = arguments.get("url");
          speechModel = arguments.get("speechModel");

          speechService = initSpeechToTextService(speech_text_api_key,speech_text_url);

            microphoneHelper = new MicrophoneHelper(this.attachedActivity);
            if (microphoneHelper != null) {
                capture = microphoneHelper.getInputStream(true);
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            speechService.recognizeUsingWebSocket(getRecognizeOptions(capture),
                                    new MicrophoneRecognizeDelegate());
                        } catch (Exception e) {
                            WatsonSttPlugin.this.events.error("error", "recognizeUsingWebSocket", null);
                        }
                    }
                }).start();
            }
        } else if (call.method.equals("stopRecognize")) {
            if (microphoneHelper != null) {
                microphoneHelper.closeInputStream();
            }
        }
        result.success(null);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    }

    @Override
    public void onListen(Object arguments, EventChannel.EventSink events) {
        this.events = events;
    }

    @Override
    public void onCancel(Object arguments) {
        System.out.println("----------------onCancel is called----------");
        this.events = null;
        microphoneHelper.closeInputStream();
    }
}
